-- Lua Switch Mapping widget by RonRon on RCGroups
-- requires at least Ethos 1.1.0 alpha 12

-- Changelog:
--  1.0  Initial version by ronron on RCGroups that allows to predefine text
--       descriptions in the source .lua file.
--  1.1  Minor additions by xStatiCa on RCGroups to add defining the text
--       descriptions on the transmitter when configuring the widget.  The info
--       is saved so that the transmitter will keep the information.  This also
--       allows each model to have it's own configuration.Version 1.1
--  1.2  Fix for reversed LS and RS in some lines
--  1.3  Added option to display switch names so that only the description is configured
--       Added option to display switch lines that have empty discriptions or not
--  1.4  Changes for Ethos 1.1.0 alpha 8  (changed lcd.setFont to lcd.font and lcd.setColor with lcd.color)
--  1.5  Changes for Ethos 1.1.0 alpha 12 (changed storage functions to alpha 12 storage functions)
--  1.6  Moved script into a subfolder and includes image in the new folder so that you don't have to put it in the global bitmaps directory separately
--  1.7  Added check and text to display "Unsupported widget size" if you use a widget other than a large widget.
--  1.8  Changed text for SM to POT3 which is the real name of the slider in the middle bottom of the transmitter
--  1.9  Fixed bug where Middle Slider (Pot3) info was not properly stored/read (by Alfred Vink)
--       Added RIGHT alignment text for RH switches and CENTERED alignment for text for middle slider (POT3) (by Alfred Vink)
--  1.10 Fixed bug where "SE "was shown even if Display Switch Names was off (by Alfred Vink) 

local function create()
	return {DisplayEmpty=true, DisplaySwitchNames=true, SAtext="", SBtext="", SCtext="", SDtext="", SEtext="", SFtext="", SGtext="", SHtext="", SItext="", SJtext="", LStext="", RStext="", MStext="", POT1text="", POT2text=""}
end

local mask
local bitmap3

local function paint(widget)
    -- Demo of a mask
	widget_w, widget_h = lcd.getWindowSize()
	if( widget_w ~= 784 or widget_h ~= 294 ) then
		lcd.drawText( 0, 0, "Unsupported widget size" )
		return
	end
	
    lcd.color( lcd.RGB(155, 155, 0) )
    lcd.drawMask(150, 25, mask)

    lcd.drawBitmap(1,1,bitmap3,widget_w-10,widget_h-20)   
    local prefixtext
	lcd.color(WHITE)
	lcd.font(FONT_S)
 ----LEFT SIDE------
	if widget.DisplayEmpty or widget.SAtext ~= "" then
		if widget.DisplaySwitchNames == true then
			prefixtext = "SA "
		else
			prefixtext = ""
		end

		lcd.drawText(5, 55, prefixtext .. widget.SAtext)
		lcd.drawLine(5,70,180,70)
	end 
	if widget.DisplayEmpty or widget.SBtext ~= "" then
		if widget.DisplaySwitchNames == true then
			prefixtext = "SB "
		else
			prefixtext = ""
		end

		lcd.drawText(5, 80, prefixtext .. widget.SBtext)
		lcd.drawLine(5,95,220,95)
		 lcd.drawLine(220,95,230,85)
	end
	if widget.DisplayEmpty or widget.SEtext ~= "" then
		if widget.DisplaySwitchNames == true then
			prefixtext = "SE "
		else
			prefixtext = ""
		end

		lcd.drawText(5, 25, prefixtext .. widget.SEtext)
		lcd.drawLine(5,40,200,40)
	end 
	if widget.DisplayEmpty or widget.SFtext ~= "" then
		if widget.DisplaySwitchNames == true then
			prefixtext = "SF "
		else
			prefixtext = ""
		end

		lcd.drawText(5, 5, prefixtext .. widget.SFtext)
		lcd.drawLine(5,20,245,20)
		 lcd.drawLine(245,20,250,25)
	end
	if widget.DisplayEmpty or widget.SItext ~= "" then
		if widget.DisplaySwitchNames == true then
			prefixtext = "SI "
		else
			prefixtext = ""
		end

		lcd.drawText(5, 105, prefixtext .. widget.SItext)
		lcd.drawLine(5,120,150,120)
	end 
	if widget.DisplayEmpty or widget.LStext ~= "" then
		if widget.DisplaySwitchNames == true then
			prefixtext = "LS "
		else
			prefixtext = ""
		end

		lcd.drawText(5, 135, prefixtext .. widget.LStext)
		lcd.drawLine(5,150,180,150)
		lcd.drawLine(180,150,200,130)
	end 
	if widget.DisplayEmpty or widget.POT1text ~= "" then
		if widget.DisplaySwitchNames == true then
			prefixtext = "POT1 "
		else
			prefixtext = ""
		end

		lcd.drawText(280, 45, prefixtext .. widget.POT1text)
		lcd.drawLine(280,60,360,60)
		lcd.drawLine(320,60,330,70)
	end 
 ------RIGHT SIDE------
	if widget.DisplayEmpty or widget.SCtext ~= "" then
		if widget.DisplaySwitchNames == true then
			prefixtext = "SC "
		else
			prefixtext = ""
		end

		lcd.drawText(780, 55, prefixtext .. widget.SCtext, RIGHT)
		lcd.drawLine(580,70,780,70)
	end 
	if widget.DisplayEmpty or widget.SDtext ~= "" then
		if widget.DisplaySwitchNames == true then
			prefixtext = "SD "
		else
			prefixtext = ""
		end

		lcd.drawText(780, 80, prefixtext .. widget.SDtext, RIGHT)
		lcd.drawLine(550,95,780,95)
		lcd.drawLine(550,95,540,85)
	end
	if widget.DisplayEmpty or widget.SHtext ~= "" then
		if widget.DisplaySwitchNames == true then
			prefixtext = "SH "
		else
			prefixtext = ""
		end

		lcd.drawText(780, 25, prefixtext .. widget.SHtext, RIGHT)
		lcd.drawLine(565,40,780,40)
	end 
	if widget.DisplayEmpty or widget.SGtext ~= "" then
		if widget.DisplaySwitchNames == true then
			prefixtext = "SG "
		else
			prefixtext = ""
		end

		lcd.drawText(780, 5, prefixtext .. widget.SGtext, RIGHT)
		lcd.drawLine(520,20,780,20)
		 lcd.drawLine(520,20,515,25)
	end
	if widget.DisplayEmpty or widget.SJtext ~= "" then
		if widget.DisplaySwitchNames == true then
			prefixtext = "SJ "
		else
			prefixtext = ""
		end

		lcd.drawText(780, 105, prefixtext .. widget.SJtext, RIGHT)
		lcd.drawLine(620,120,780,120)
	end 
	if widget.DisplayEmpty or widget.RStext ~= "" then
		if widget.DisplaySwitchNames == true then
			prefixtext = "RS "
		else
			prefixtext = ""
		end

		lcd.drawText(780, 135, prefixtext .. widget.RStext, RIGHT)
		lcd.drawLine(590,150,780,150)
		lcd.drawLine(590,150,570,130)
	end 
	if widget.DisplayEmpty or widget.POT2text ~= "" then
		if widget.DisplaySwitchNames == true then
			prefixtext = "POT2 "
		else
			prefixtext = ""
		end

		lcd.drawText(480, 45, prefixtext .. widget.POT2text, RIGHT)
		lcd.drawLine(400,60,480,60)
		lcd.drawLine(440,60,430,70)
	end 
	if widget.DisplayEmpty or widget.MStext ~= "" then
		if widget.DisplaySwitchNames == true then
			prefixtext = "POT3 "
		else
			prefixtext = ""
		end

		lcd.drawText(385, 240, prefixtext .. widget.MStext, CENTERED)
		lcd.drawLine(350,240,420,240)
		lcd.drawLine(385,240,385,220)
	end
end

local function configure(widget)
	local line = form.addLine("Display empty?")
	form.addBooleanField(line, nil, function() return widget.DisplayEmpty end, function(value) widget.DisplayEmpty = value end)
	line = form.addLine("Display switch names?")
	form.addBooleanField(line, nil, function() return widget.DisplaySwitchNames end, function(value) widget.DisplaySwitchNames = value end)
	line = form.addLine("Switch SA")
	form.addTextField(line, nil, function() return widget.SAtext end, function(value) widget.SAtext = value end)
	line = form.addLine("Switch SB")
	form.addTextField(line, nil, function() return widget.SBtext end, function(value) widget.SBtext = value end)
	line = form.addLine("Switch SC")
	form.addTextField(line, nil, function() return widget.SCtext end, function(value) widget.SCtext = value end)
	line = form.addLine("Switch SD")
	form.addTextField(line, nil, function() return widget.SDtext end, function(value) widget.SDtext = value end)
	line = form.addLine("Switch SE")
	form.addTextField(line, nil, function() return widget.SEtext end, function(value) widget.SEtext = value end)
	line = form.addLine("Switch SF")
	form.addTextField(line, nil, function() return widget.SFtext end, function(value) widget.SFtext = value end)
	line = form.addLine("Switch SG")
	form.addTextField(line, nil, function() return widget.SGtext end, function(value) widget.SGtext = value end)
	line = form.addLine("Switch SH")
	form.addTextField(line, nil, function() return widget.SHtext end, function(value) widget.SHtext = value end)
	line = form.addLine("Switch SI")
	form.addTextField(line, nil, function() return widget.SItext end, function(value) widget.SItext = value end)
	line = form.addLine("Switch SJ")
	form.addTextField(line, nil, function() return widget.SJtext end, function(value) widget.SJtext = value end)
	line = form.addLine("Slider LS")
	form.addTextField(line, nil, function() return widget.LStext end, function(value) widget.LStext = value end)
	line = form.addLine("Slider RS")
	form.addTextField(line, nil, function() return widget.RStext end, function(value) widget.RStext = value end)
	line = form.addLine("Slider middle POT3")
	form.addTextField(line, nil, function() return widget.MStext end, function(value) widget.MStext = value end)
	line = form.addLine("Knob POT1")
	form.addTextField(line, nil, function() return widget.POT1text end, function(value) widget.POT1text = value end)
	line = form.addLine("Knob POT2")
	form.addTextField(line, nil, function() return widget.POT2text end, function(value) widget.POT2text = value end)
end

local function read(widget)
    widget.SAtext = storage.read("string")
	widget.SBtext = storage.read("string")
	widget.SCtext = storage.read("string")
	widget.SDtext = storage.read("string")
	widget.SEtext = storage.read("string")
	widget.SFtext = storage.read("string")
	widget.SGtext = storage.read("string")
	widget.SHtext = storage.read("string")
	widget.SItext = storage.read("string")
	widget.SJtext = storage.read("string")
	widget.LStext = storage.read("string")
	widget.RStext = storage.read("string")
	widget.MStext = storage.read("string")
	widget.POT1text = storage.read("string")
	widget.POT2text = storage.read("string")
	widget.DisplayEmpty = storage.read("boolean")
	widget.DisplaySwitchNames = storage.read("boolean")
end

local function write(widget)
    storage.write("string", widget.SAtext)
	storage.write("string", widget.SBtext)
	storage.write("string", widget.SCtext)
	storage.write("string", widget.SDtext)
	storage.write("string", widget.SEtext)
	storage.write("string", widget.SFtext)
	storage.write("string", widget.SGtext)
	storage.write("string", widget.SHtext)
	storage.write("string", widget.SItext)
	storage.write("string", widget.SJtext)
	storage.write("string", widget.LStext)
	storage.write("string", widget.RStext)
	storage.write("string", widget.MStext)
	storage.write("string", widget.POT1text)
	storage.write("string", widget.POT2text)
	storage.write("boolean", widget.DisplayEmpty)
	storage.write("boolean", widget.DisplaySwitchNames)
end

function clearallfields(widget)

end

local function init()
    mask = lcd.loadMask("FLASH:/bitmaps/system/mask_dot.png")
    system.registerWidget({key="swmap", name="Switch Mapping", create=create, paint=paint, configure=configure, read=read, write=write})
    bitmap3 = lcd.loadBitmap("/scripts/swmap/bitmaps/x203.png")
end

return {init=init}
